import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:momomotus/cubit/game_cubit.dart';
import 'package:momomotus/models/game/game.dart';

class GameCellWidget extends StatelessWidget {
  const GameCellWidget({
    super.key,
    required this.cellValue,
    required this.cellTip,
    required this.hasFocus,
    required this.foundLetter,
  });

  final String cellValue;
  final String cellTip;
  final bool hasFocus;
  final String foundLetter;

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<GameCubit, GameState>(
      builder: (BuildContext context, GameState gameState) {
        final Game currentGame = gameState.currentGame;

        final String skin = currentGame.globalSettings.skin;

        const Color textColor = Colors.white;
        final Color focusBorderColor = Colors.yellow.shade700;
        const Color defaultBorderColor = Colors.white;

        String cellImage = 'empty';
        if (cellTip != '') {
          cellImage = cellTip;
        }

        String displayedCellValue = cellValue;

        if ((foundLetter != ' ') && (cellValue == ' ')) {
          displayedCellValue = foundLetter;
          cellImage = 'good';
        }

        final Image imageWidget = Image(
          image: AssetImage('assets/skins/${skin}_$cellImage.png'),
          fit: BoxFit.fill,
        );

        final Widget cellBackground = Container(
          decoration: BoxDecoration(
            border: Border.all(
              width: 4.0,
              color: hasFocus ? focusBorderColor : defaultBorderColor,
              style: BorderStyle.solid,
            ),
          ),
          child: imageWidget,
        );

        Text textWidget = Text(
          displayedCellValue,
          style: const TextStyle(
            color: textColor,
            fontSize: 40.0,
            fontWeight: FontWeight.w900,
          ),
          textAlign: TextAlign.center,
        );

        return Stack(
          alignment: Alignment.center,
          children: <Widget>[
            cellBackground,
            Center(child: textWidget),
          ],
        );
      },
    );
  }
}
